from netbox_proxmox_sync.api.config import NB_API
from netbox_proxmox_sync.api.utils.errors import APIError, ValidationError
from netbox_proxmox_sync.api.proxmox.extract import Proxmox


def cluster(proxmox_cluster):
    cluster_name = proxmox_cluster['name']
    try:
        netbox_clusters = NB_API.virtualization.clusters.filter(name=cluster_name)
    except Exception as e:
        raise APIError(e)
    if (len(netbox_clusters)) == 0:
        raise ValidationError(f'Cluster "{cluster_name}" does not exist!')
    netbox_cluster = list(netbox_clusters)[0]
    # Return JSON serializable dict :)
    return dict(netbox_cluster)


def tags():
    try:
        netbox_tags = NB_API.extras.tags.filter(content_types=[
            'virtualization.virtualmachines',
        ])
    except Exception as e:
        raise APIError(e)
    # Return JSON serializable list :)
    return [dict(tag) for tag in netbox_tags]


def custom_fields():
    try:
        netbox_custom_fields = NB_API.extras.custom_fields.filter(
            name='vmid',
            object_types='virtualization.virtualmachine',
        )
    except Exception as e:
        raise APIError(e)
    # Return JSON serializable list :)
    return [dict(custom_field) for custom_field in netbox_custom_fields]


def vms_and_interfaces(netbox_cluster):
    try:
        cluster_id = netbox_cluster['id']
        netbox_vms = NB_API.virtualization.virtual_machines.filter(cluster_id=cluster_id)
        netbox_interfaces = NB_API.virtualization.interfaces.filter(cluster_id=cluster_id)
    except Exception as e:
        raise APIError(e)
    # Return JSON serializable lists :)
    return [dict(x) for x in netbox_vms], [dict(x) for x in netbox_interfaces]


def delete_stuff(cluster, tags, custom_fields, vms, interfaces):
    # This order is important
    rt = NB_API.extras.tags.delete([tag['id'] for tag in tags])
    rc = NB_API.extras.custom_fields.delete([field['id'] for field in custom_fields])
    ri = NB_API.virtualization.interfaces.delete([i['id'] for i in interfaces])
    rv = NB_API.virtualization.virtual_machines.delete([vm['id'] for vm in vms])
    rf = NB_API.virtualization.clusters.delete([cluster['id']])
    errors = []
    if rt is None:
        errors.append('Could not delete tags!')
    if rf is None:
        errors.append('Could not delete custom_fields!')
    if rc is None:
        errors.append('Could not delete cluster!')
    if rv is None:
        errors.append('Could not delete virtual machines!')
    if ri is None:
        errors.append('Could not delete interfaces!')
    if len(errors) > 0:
        raise APIError('Could not delete plugin information!', errors=errors)


def all():
    proxmox_cluster, _ = Proxmox.fetch_cluster_and_nodes()
    netbox_cluster = cluster(proxmox_cluster)
    netbox_tags = tags()
    netbox_custom_fields = custom_fields()
    netbox_vms, netbox_interfaces = vms_and_interfaces(netbox_cluster)
    delete_stuff(
        netbox_cluster, netbox_tags, netbox_custom_fields,
        netbox_vms, netbox_interfaces
    )
    return {
        'cluster': netbox_cluster,
        'tags': netbox_tags,
        'custom_fields': netbox_custom_fields,
        'vms': netbox_vms,
        'interfaces': netbox_interfaces,
    }
