from netbox_proxmox_sync.api.config import (
    NB_API,
    NETBOX_SITE_ID,
    NETBOX_CLUSTER_TYPE_ID,
)
from netbox_proxmox_sync.api.utils.errors import APIError
from netbox_proxmox_sync.api.utils.models import (
    tag_from_netbox,
    vm_from_netbox,
    interface_from_netbox,
)


class NetBox:
    @staticmethod
    def fetch_cluster(cluster_name):
        try:
            netbox_cluster = NB_API.virtualization.clusters.filter(name=cluster_name)
        except Exception:
            raise APIError('Failed to connect to Proxmox Cluster!')
        return {
            'id': netbox_cluster.id,
            'name': netbox_cluster.name,
            'type': NETBOX_CLUSTER_TYPE_ID,
            'description': netbox_cluster.description,
            'site': NETBOX_SITE_ID,
        }

    @staticmethod
    def fetch_tags():
        try:
            netbox_tags = [dict(tag) for tag in NB_API.extras.tags.filter(
                content_types='virtualization.virtualmachines',
            )]
        except Exception as e:
            raise APIError(e)

        return [tag_from_netbox(tag) for tag in netbox_tags]

    @staticmethod
    def fetch_virtual_machines_and_interfaces(cluster_id):
        try:
            netbox_vms = [
                dict(vm) for vm in NB_API.virtualization.virtual_machines.filter(
                    cluster_id=cluster_id
                )
            ]
            netbox_interfaces = [dict(i) for i in NB_API.virtualization.interfaces.all()]
        except Exception as e:
            raise APIError(e)

        vms = [vm_from_netbox(vm) for vm in netbox_vms]
        interfaces = [interface_from_netbox(i) for i in netbox_interfaces]

        return vms, interfaces
